;
; *****************************************************************************************************************
; ******************************************     Copyright Notice     *********************************************
; *                                                                                                               *
; *  This work was produced at the University of California, Lawrence Livermore National Laboratory (UC LLNL)     *
; *  under contract no. W-7405-ENG-48 (Contract 48) between the U.S. Department of Energy (DOE) and The Regents   *
; *  of the University of California (University) for the operation of UC LLNL. Copyright is reserved to the      *
; *  University for purposes of controlled dissemination, commercialization through formal licensing, or other    *
; *  disposition under terms of Contract 48; DOE policies, regulations and orders; and U.S. statutes. The rights  *
; *  of the Federal Government are reserved under Contract 48 subject to the restrictions agreed upon by the DOE  * 
; *  and University as allowed under DOE Acquisition Letter 97-1.                                                 *
; *                                                                                                               *
; *****************************************************************************************************************
;
; *****************************************************************************************************************
; **********************************************     DISCLAIMER     ***********************************************
; *                                                                                                               *
; *  This work was prepared as an account of work sponsored by an agency of the United States Government.         *
; *  Neither the United States Government nor the University of California nor any of their employees, makes      *
; *  any warranty, express or implied, or assumes any liability or responsibility for the accuracy, completeness, *
; *  or usefulness  *of any information, apparatus, product, or process disclosed, or represents that its use     *
; *  would not infringe privately-owned rights.  Reference herein to any specific commercial products, process,   *
; *  or service by trade name, trademark, manufacturer or otherwise does not necessarily constitute or imply its  *
; *  endorsement, recommendation, or favoring by the United States Government or the University of California.    *
; *  The views and opinions of authors expressed herein do not necessarily state or reflect those of the United   *
; *  States Government or the University of California, and shall not be used for advertising or product          *
; *  endorsement purposes.                                                                                        *
; *                                                                                                               *
; *****************************************************************************************************************
;

;
; Defining routine for 4-dimensional signal objects
;
; Written by W.M. Nevins
;	1/31/00
;
FUNCTION GKVs4D::AxisNumber, stringIn, Debug=d
;
; Returns number of axis whose mnemonic matches 'stringIn'
; If there is no match, returns 0
; 
; Written by W.M. Nevins
;	2/20/00
;
debug=0
result=0
IF(N_ELEMENTS(d) NE 0) THEN debug=d
result = self -> GKVs3D::AxisNumber(stringIn, debug=debug)
axis4Mnemonic = STRTRIM(self.Grid4.mnemonic, 2)
IF( STRCMP(stringIn, axis4Mnemonic, /FOLD_CASE) ) THEN result = 4
RETURN, result
END ; ****** GKVs4D::AxisNumber ****** ;


FUNCTION GKVs4D::GetAxis, structure, Nstructure, Debug=d
;
; Searches 'structure' for tags which are the same as the mnemonics of axis1, axis2, axis3, and axis4
; Returns a structure with tags 'axis1', 'axis2', 'axis3', and 'axis4'.  
; The associated values are whatever values were associated with the corresponding mnemonic.
; 
; Written by W.M. Nevins
;	2/20/00
;
debug=0
otherTags = 0
IF(N_ELEMENTS(d) NE 0) THEN debug=d
;
; Check for axis1, axis2, and axis3 mnemonics by calling GKVs3D::GetAxis
;
result3 = self -> GKVs3D::GetAxis(structure, Debug=debug)
;
; Structure now contains input structure with tag to mnemonic of axis1, axis2 and axis3 (if found) deleted
;
result = {axis1:result3.axis1, axis2:result3.axis2, axis3:result3.axis3, axis4:'no match'}
IF(N_ELEMENTS(structure) EQ 0) THEN RETURN, result			; Structure is undefined
IF(TypeOf(structure) EQ 2) THEN RETURN, result				; No tags were left to return
nTags = N_TAGS(structure)
IF(nTags EQ 0) THEN RETURN, result
tagNames = TAG_NAMES(structure)
tagNames = STRTRIM(tagNames, 2)							; Remove both leading and trailing blanks
command_str = 'structure = {'
axisMnemonic = STRTRIM(self.Grid4.mnemonic)				; Remove both leading and trailing blanks
FOR i=0, ntags-1 DO BEGIN								; Search tags of 'structure'
	IF( STRCMP(axisMnemonic, tagNames[i], /FOLD_CASE) ) THEN BEGIN
		result = {axis1:result3.axis1, axis2:result3.axis2, axis3:result3.axis3, axis4:structure.(i) }	
												; If multiple occurances of axis4 mnemonic in 'structure'
	ENDIF ELSE BEGIN								;	only the LAST occurance is signifacant
		i_str = STRING(i, FORMAT='(I3)')				; Construct command string for 'Nstructure'
		i_str = STRTRIM(i_str, 2)						; Remove both leading and trailing blanks
		IF (otherTags NE 0) THEN command_str = command_str + ', '
		command_str = command_str + tagNames[i] + ':' + 'structure.(' + i_str + ')'
		otherTags = otherTags + 1
	ENDELSE
ENDFOR
command_str = command_str + '}'
IF(otherTags NE 0) THEN BEGIN
	ok = EXECUTE(command_str)
ENDIF ELSE BEGIN
	structure = -1
ENDELSE
RETURN, result
END ; ****** GKVs4D::GetAxis ****** ;

FUNCTION GKVs4D::AxisIndex, axisNumber, value, Debug=d
;
; We should only get here if axisNumber is  4.  
; Function returns index of axisNumber^th  Grid closest to 'value'
;
debug=0
IF(N_ELEMENTS(d) NE 0) then debug=d
IF(axisNumber GT 4) THEN BEGIN
	MESSAGE, 'axisNumber  4', INFORMATIONAL=d
	RETURN, 0
ENDIF
axisStr = STRING(axisNumber,  FORMAT='(I1)')
commandStr = 'temp = (self.Grid' + axisStr + '.values - values)^2'
ok = EXECUTE(commandStr)
eps=MIN(temp,index)
RETURN, index
END ; ****** GKVs4D::AxisIndex ****** ;


FUNCTION GKVs4D::GetValues, All=all, open=open
;
; Returns pointer to an array containing values which fall within the 
; signal window of 'self'
;
;	Written by W.M. Nevins
;	2/6/00
;
iimax = N_ELEMENTS(*self.Grid1.values) - 1
IF(KEYWORD_SET(open)) THEN BEGIN
	IF(self.grid1.boundary EQ "periodic (closed)") THEN iimax=iimax-1 
ENDIF
imin = self.Grid1.irange[0]
imax = self.Grid1.irange[1] < iimax
IF N_ELEMENTS(all) THEN BEGIN
	imin = 0
	imax = iimax
ENDIF

jjmax = N_ELEMENTS(*self.Grid2.values) - 1
IF(KEYWORD_SET(open)) THEN BEGIN
	IF(self.grid2.boundary EQ "periodic (closed)") THEN jjmax=jjmax-1 
ENDIF
jmin = self.Grid2.irange[0]
jmax = self.Grid2.irange[1] < jjmax
IF N_ELEMENTS(all) THEN BEGIN
	jmin = 0
	jmax = jjmax
ENDIF

kkmax = N_ELEMENTS(*self.Grid3.values) - 1
IF(KEYWORD_SET(open)) THEN BEGIN
	IF(self.grid3.boundary EQ "periodic (closed)") THEN kkmax=kkmax-1 
ENDIF
kmin = self.Grid3.irange[0]
kmax = self.Grid3.irange[1] < kkmax
IF N_ELEMENTS(all) THEN BEGIN
	kmin = 0
	kmax = kkmax
ENDIF

llmax = N_ELEMENTS(*self.Grid4.values) - 1
IF(KEYWORD_SET(open)) THEN BEGIN
	IF(self.grid4.boundary EQ "periodic (closed)") THEN llmax=llmax-1 
ENDIF
lmin = self.Grid4.irange[0]
lmax = self.Grid4.irange[1] < llmax
IF N_ELEMENTS(all) THEN BEGIN
	lmin = 0
	lmax = llmax
ENDIF
values = (*self.values)[imin:imax, jmin:jmax, kmin:kmax, lmin:lmax]
RETURN, PTR_NEW(values)
END ; ****** GKVs4D::GetValues ******


FUNCTION GKVs4D::GetErrors, All=all
;
; Returns pointer to an array containing errorBars which fall within the 
; signal window of 'self'
;
;	Written by W.M. Nevins
;	10/10/00
;
imin = self.Grid1.irange[0]
imax = self.Grid1.irange[1]
IF N_ELEMENTS(all) THEN BEGIN
	imin = 0
	imax = N_ELEMENTS(self.Grid1.values) - 1
ENDIF
jmin = self.Grid2.irange[0]
jmax = self.Grid2.irange[1]
IF N_ELEMENTS(all) THEN BEGIN
	jmin = 0
	jmax = N_ELEMENTS(self.Grid2.values) - 1
ENDIF
kmin = self.Grid3.irange[0]
kmax = self.Grid3.irange[1]
IF N_ELEMENTS(all) THEN BEGIN
	kmin = 0
	kmax = N_ELEMENTS(self.Grid3.values) - 1
ENDIF
lmin = self.Grid4.irange[0]
lmax = self.Grid4.irange[1]
IF N_ELEMENTS(all) THEN BEGIN
	lmin = 0
	lmax = N_ELEMENTS(self.Grid4.values) - 1
ENDIF
errors = (*self.ErrorBars)[imin:imax, jmin:jmax, kmin:kmax, lmin:lmax]
RETURN, PTR_NEW(errors)
END ; ****** GKVs4D::GetErrors ******


FUNCTION GKVs4D::SameGrid, argObj, All=all
;
; Returns 1 if 'self' and argObj have same grid, 0 otherwise
;
;	Written 2/6/00
;	by W.M. Nevins
;
selfDims = self  -> NumDims()
argDims  = argObj-> NumDims()
IF(argDims NE selfDims) THEN RETURN, 0
imin = self.Grid4.irange[0]
imax = self.Grid4.irange[1]
IF N_ELEMENTS(all) THEN BEGIN
	imin = 0
	imax = N_ELEMENTS(*self.Grid4.values) - 1
ENDIF
sGrid = (*self.Grid4.values)[imin:imax]
iargmin = argObj.Grid4.irange[0]
iargmax = argObj.Grid4.irange[1]
IF N_ELEMENTS(all) THEN BEGIN
	iargmin = 0
	iargmax = N_ELEMENTS(*argObj.Grid4.values) - 1
ENDIF
IF((imax-imin) NE (iargmax-iargmin)) THEN RETURN, 0
argGrid = (*argObj.Grid4.values)[iargmin:iargmax]
Err = TOTAL((sGrid - argGrid)^2)/(1+imax-imin)
L1 = sGrid[imax-imin] - sGrid[0]
d1 = L1/(1+imax-imin)
IF(ERR LT 1.e-3*(d1^2)) THEN RETURN, self -> GKVs3D::SameGrid(argObj)
RETURN, 0
END ; ****** GKVs4D::SameGrid ****** ;


FUNCTION GKVs4D::Interpolate, arg
;
; Interpolate values from self onto the signal window of arg's grid.
;
IF (OBJ_ISA(arg, 'GKVs4D') NE 1) THEN BEGIN
	MESSAGE, "Argument is not a valid GKVs4D object"
	RETURN, 0
ENDIF
;
; Check for common units, independent variable, interval
;
IF (	(self.Grid1.units NE arg.Grid1.units) OR 	$
	(self.Grid2.units NE arg.Grid2.units) OR	$
	(self.Grid3.units NE arg.Grid3.units) OR	$
	(self.Grid4.units NE arg.Grid4.units)		) 	THEN BEGIN
	MESSAGE, "Incompatible units (independent variables)"
	RETURN, 0
ENDIF
IF (	(self.Grid1.title NE arg.Grid1.title) OR 	$
	(self.Grid2.title NE arg.Grid2.title) OR	$
	(self.Grid3.title NE arg.Grid3.title) OR	$
	(self.Grid4.units NE arg.Grid4.units)		) 	THEN BEGIN
	MESSAGE, "Interpolate:  Incompatible independent variables"
	RETURN, 0
ENDIF
IF(self -> SameGRID(arg)) THEN BEGIN
	MESSAGE, "Objects have common grid, no interpolation necessary", /Informational
	result = self -> MakeCopy()
	result -> Restrict
	RETURN, result
ENDIF
MESSAGE, 'Interpolation not yet implimented in 4D', /Informational
RETURN, 0
END ; ****** GKVs4D::Interpolate ****** ;


FUNCTION GKVs4D::MakeCopy, _Extra=extra
;
; Make "deep" copy of self
;
result = self -> GKVs3D::MakeCopy( _Extra=extra)	; Creates new GKVsxD objects, and
										; 	copies elements of GKVs3D class
result.Grid4 = GKVsd_GridCopy(self.Grid4)			; Make 'deep' copy the Grid-class structure Grid4,			
RETURN, result

END ; ***** GKVs4D::MakeCopy ***** ;


FUNCTION GKVs4D_Gen, Nx=n1, Ny=n2, Nz=n3, Nt=n4, Amplitude=a,  	$
				kx=k1, ky=k2, kz=k3, omega=k4, Del_k=bandwidth
;
; Set up sample GKVs2D signal object
;
nx=32L
ny=32L
nz=32L
nt=32L
IF ( N_ELEMENTS(n1) NE 0 ) THEN nx=n1
IF ( N_ELEMENTS(n2) NE 0 ) THEN ny=n2
IF ( N_ELEMENTS(n3) NE 0 ) THEN nz=n3
IF ( N_ELEMENTS(n4) NE 0 ) THEN nt=n4
dx = 2*!PI/(nx-1)
dy = 2*!PI/(ny-1)
dz = 2*!PI/(nz-1)
dt = 2*!PI/(nt-1)

amplitude=1.
IF KEYWORD_SET(a)  then amplitude=a

kx = 2.
ky = 2.
kz = 2.
omega = 2.
Del_k = 2.
IF N_ELEMENTS(k1) THEN kx=k1
IF N_ELEMENTS(k2) THEN ky=k2
IF N_ELEMENTS(k3) THEN kz=k3
IF N_ELEMENTS(k4) THEN omega=k4
IF N_ELEMENTS(bandwidth) then del_k=bandwidth

xgrid = dx*indgen(nx)
ygrid = dy*indgen(ny)
zgrid = dz*indgen(nz)
tgrid = dt*indgen(nt)
i=COMPLEX(0.,1.)
argx= -0.5d*((INDGEN(nx) - kx)^2/Del_k > (-30.0d))
argy= -0.5d*((INDGEN(ny) - ky)^2/Del_k > (-30.0d))
argz= -0.5d*((INDGEN(nz) - kz)^2/Del_k > (-30.0d))
argt= -0.5d*((INDGEN(nt) - omega)^2/Del_k > (-30.0d))
rarray=RANDOMN(seed, nx, ny, nz, nt)
iarray=RANDOMN(seed, nx, ny, nz, nt)
array=COMPLEX(rarray, iarray)
FOR i=0, nz-1 DO	$
	FOR j=0, nt-1 DO $
		array(*,*,i,j)=array(*,*,i,j)*( exp(argx)#exp(argy) )*( exp(argz(i))*exp(argt(j)) )
array=FFT(array, 1, /OVERWRITE)
msarray= TOTAL(array*CONJ(array))/( LONG(nx)*LONG(ny)*LONG(nz)*LONG(nt) ) 
values=amplitude*array/sqrt(msarray)
vmin = GKVsd_MIN(values, MAX=vmax)
IF(vmax EQ vmin) THEN vmax = vmin+1

Indices = REPLICATE('*', 4)

signal={GKVs4D}
;
; GKVsd tags
;
	signal.mnemonic	= "sgen4D"
	signal.Title	= "!13Title!N!X"
	signal.Indices	= PTR_NEW(Indices)
	signal.units	= "units"
	signal.values	= ptr_new(values)
	signal.vrange	= [vmin, vmax]
	signal.codeName	= "GKV selftest"
	signal.codePI	= "W.M. Nevins"	
	signal.RunID	= "Null run"
	signal.FileID	= "Self-Generated"
;
; Grid1 tags
;
	signal.Grid1.mnemonic	= 'x'
	signal.Grid1.title 	= "x-title"
	signal.Grid1.units 	= "x-units"
	signal.Grid1.values  = ptr_new(xgrid)
	signal.Grid1.boundary	= "open"
	signal.Grid1.range 	= [xgrid[0], xgrid[nx-1]]
	signal.Grid1.irange 	= [0, nx-1]
;
; Grid2 tags
;
	signal.Grid2.mnemonic	= 'y'
	signal.Grid2.title 	= "y-title"
	signal.Grid2.units 	= "y-units"
	signal.Grid2.values 	= PTR_NEW(ygrid)
	signal.Grid2.boundary	= "open"
	signal.Grid2.range	= [ygrid[0], ygrid[ny-1]]
	signal.Grid2.irange	= [0, ny-1]
;
; Grid3 tags
;
	signal.Grid3.mnemonic	= 'z'
	signal.Grid3.title 	= "z-title"
	signal.Grid3.units 	= "z-units"
	signal.Grid3.values 	= PTR_NEW(zgrid)
	signal.Grid3.boundary	= "open"
	signal.Grid3.range	= [zgrid[0], zgrid[nz-1]]
	signal.Grid3.irange	= [0, nz-1]
;
; Grid4 tags
;
	signal.Grid4.mnemonic	= 't'
	signal.Grid4.title 	= "t-title"
	signal.Grid4.units 	= "t-units"
	signal.Grid4.values 	= PTR_NEW(tgrid)
	signal.Grid4.boundary	= "open"
	signal.Grid4.range	= [tgrid[0], tgrid[nt-1]]
	signal.Grid4.irange	= [0, nt-1]
;
; Create signal object
;
	signal_obj=OBJ_NEW("GKVs4D", signal) 
RETURN, signal_obj
END ; ***** GKVs4D_Gen ***** ;


PRO GKVs4D::GET, axis=axisID,  GridMnemonic = GridMnemonic, GridTitle=GridTitle, 				$
			GridUnits=GridUnits, GridValues=GridValues, boundary=boundary, uniform=uniform,		$
			range=range, irange=irange, mnemonic=mnemonic, Title=Title, Indices=indices, 		$
			units=units, values=values, vrange=vrange, ErrorBars=ErrorBars, CodeName=CodeName,	$
			CodePI=CodePI, RunID=RunID, FileID=FIleID, _Extra=extra	
;
; Get values of elements of the realization of the GRID class, Grid4;
; and then call GKVs3D::GET to get values of elements of the GKVs3D Class.
;
; This call will be used polymorphically, so 'self' may be of class GKVs4D,
; or any or its subclasses
;
IDinfo = SIZE(axisID)
IDtype = IDinfo[IDinfo[0] + 1]
CASE IDtype OF
	1:	IF(axisID EQ 4) THEN 	$
			GKVsd_GetGrid, self.Grid4,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
								GridUnits=GridUnits,	GridValues=GridValues, 			$
								boundary=boundary, uniform=uniform,range=range, 		$
								irange=irange, _Extra=extra
	2:	IF(axisID EQ 4) THEN 	$
			GKVsd_GetGrid, self.Grid4,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
								GridUnits=GridUnits,	GridValues=GridValues, 			$
								boundary=boundary, uniform=uniform,range=range, 		$
								irange=irange, _Extra=extra
	3:	IF(axisID EQ 4) THEN 	$
			GKVsd_GetGrid, self.Grid4,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
								GridUnits=GridUnits,	GridValues=GridValues, 			$
								boundary=boundary, uniform=uniform,range=range, 		$
								irange=irange, _Extra=extra
	7:	IF(axisID EQ self.Grid4.mnemonic) THEN 	$
			GKVsd_GetGrid, self.Grid4,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
								GridUnits=GridUnits,	GridValues=GridValues, 			$
								boundary=boundary, uniform=uniform,range=range, 		$
								irange=irange, _Extra=extra
ELSE	 :	; just continue on else 
ENDCASE
self -> GKVs3D::GET, axis=axisID,  GridMnemonic = GridMnemonic, GridTitle=GridTitle, 				$
			GridUnits=GridUnits, GridValues=GridValues, boundary=boundary, uniform=uniform,		$
			range=range, irange=irange, mnemonic=mnemonic, Title=Title, Indices=indices, 		$
			units=units, values=values, vrange=vrange, ErrorBars=ErrorBars, CodeName=CodeName,	$
			CodePI=CodePI, RunID=RunID, FileID=FIleID, _Extra=extra
RETURN
END ; ****** GKVs4D::GET ****** ;


PRO GKVs4D::SET,axis=axisID,  GridMnemonic = GridMnemonic, GridTitle=GridTitle, 				$
			GridUnits=GridUnits, GridValues=GridValues, boundary=boundary, uniform=uniform,		$
			range=range, irange=irange, mnemonic=mnemonic, Title=Title, Indices=indices, 		$
			units=units, values=values, vrange=vrange, ErrorBars=ErrorBars, CodeName=CodeName,	$
			CodePI=CodePI, RunID=RunID, FileID=FIleID, _Extra=extra
;
; Get values of elements of the realization of the GRID class, Grid4;
; and then call GKVs3D::GET to get values of elements of the GKVs3D Class.
;
; This call will be used polymorphically, so 'self' may be of class GKVs4D,
; or any or its subclasses
;
arg = self.Grid4
IDinfo = SIZE(axisID)
IDtype = IDinfo[IDinfo[0] + 1]
CASE IDtype OF
	1:	IF(axisID EQ 4) THEN 	$
			GKVsd_SetGrid, arg,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
							GridUnits=GridUnits,	GridValues=GridValues, 			$
							boundary=boundary, uniform=uniform,range=range, 		$
							irange=irange, _Extra=extra
	2:	IF(axisID EQ 4) THEN 	$
			GKVsd_SetGrid, arg,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
							GridUnits=GridUnits,	GridValues=GridValues, 			$
							boundary=boundary, uniform=uniform,range=range, 		$
							irange=irange, _Extra=extra
	3:	IF(axisID EQ 4) THEN 	$
			GKVsd_SetGrid, arg,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
							GridUnits=GridUnits,	GridValues=GridValues, 			$
							boundary=boundary, uniform=uniform,range=range, 		$
							irange=irange, _Extra=extra
	7:	IF(axisID EQ self.Grid4.mnemonic) THEN 	$
			GKVsd_SetGrid, arg,	GridMnemonic = GridMnemonic, GridTitle=GridTitle, 		$
							GridUnits=GridUnits,	GridValues=GridValues, 			$
							boundary=boundary, uniform=uniform,range=range, 		$
							irange=irange, _Extra=extra
ELSE	 :	; just continue on else 
ENDCASE
self.Grid4 = arg
self -> GKVs3D::SET, 	axis=axisID,  GridMnemonic = GridMnemonic, GridTitle=GridTitle, 				$
			GridUnits=GridUnits, GridValues=GridValues, boundary=boundary, uniform=uniform,		$
			range=range, irange=irange, mnemonic=mnemonic, Title=Title, Indices=indices, 		$
			units=units, values=values, vrange=vrange, ErrorBars=ErrorBars, CodeName=CodeName,	$
			CodePI=CodePI, RunID=RunID, FileID=FIleID, _Extra=extra
RETURN
END ; ****** GKVs4D::SET ****** ;


PRO GKVs4D::Info
;
; Prints information about contents of GKVs4D objects
;
self -> GKVs3D::Info
PRINT, 'Grid4:'
GKVsd_PrintGrid, self.Grid4
RETURN
END ; ***** GKVs4D::Info ***** ;


Function GKVs4D::INIT, signal
;
; Preliminary init for testing object methods ...
;
Catch, error 
IF error NE 0 then Begin 
   Catch, /Cancel             ; Cancel error trap 
   ok=Error_Message(/Trace)   ; print out a trace-back in the output log 
   RETURN, 0                  ; Return 0 for unsuccessful call to INIT function. 
ENDIF 
;
; Set tags for GKVs3D class
;
ok = self -> GKVs3D::INIT(signal)
;
; Set Tags for Grid4
;
NumTags=N_TAGS(self.Grid4)
FOR itag=0, NumTags-1 DO $
	self.Grid4.(itag) = signal.Grid4.(itag)
;
; remove embedded blanks in grid title, mnemonic, and units
;
self.grid4.title    = STRCOMPRESS(self.grid4.title,    /REMOVE_ALL)
self.grid4.mnemonic = STRCOMPRESS(self.grid4.mnemonic, /REMOVE_ALL)
self.grid4.units    = STRCOMPRESS(self.grid4.units,    /REMOVE_ALL)


IF(PTR_VALID(self.Grid4.values)) THEN BEGIN
	;
	; Check for uniform grid
	;
	self.Grid4.uniform = GKVsd_UniformGrid(self.Grid4.values)
	;
	; Check if Grid4.range is set ... and set if necessary
	;
	IF((self.Grid4.range[0] EQ 0.) AND (self.Grid4.range[1] EQ 0.)) THEN $
		self.Grid4.range = [MIN(*self.Grid4.Values, Max=max), max]
	;
	; Check if Grid4.irange is set ... and set if necessary
	;
	IF((self.Grid4.irange[0] EQ 0) AND (self.Grid4.irange[1] EQ 0)) THEN $
		self.Grid4.irange = [0, N_ELEMENTS(*self.Grid4.values)-1]
ENDIF
;
; Return on successful completion
;
RETURN, ok
END ; ***** GKVs4D::INIT ***** ;


PRO GKVs4D::CleanUp


PTR_FREE, self.Grid4.values
self -> GKVs3D::CleanUp

RETURN

END ; ***** GKVs4D::CleanUp ***** ;	

PRO GKVs4D__Define
struct = {	GKVs4D,				$	; "GK Visualization signal (4-D)"
		INHERITS GKVs3D,		$	; GKVs4D is a subclass of GKVs3D
		GRID4:{GRID}			}	; Include a fourth 'Grid' class
	
END ; ***** GKVs4D__Define ***** ;
